@@ -29,6 +29,7 @@ static const char *callbacklib_names[total_callbacks] = {
     "start_run",
     "stop_run",
     "define_font",
+    "quality_font",
     "pre_output_filter",
     "buildpage_filter",
     "hpack_filter",
@@ -54,9 +55,11 @@ static const char *callbacklib_names[total_callbacks] = {
     "show_warning_message",
     "hpack_quality",
     "vpack_quality",
+    "show_break",
+    "show_build",
     "insert_par",
     "append_line_filter",
-    "build_page_insert",
+    "insert_distance",
  /* "fire_up_output", */
     "wrapup_run",
     "begin_paragraph",
@@ -74,6 +77,10 @@ static const char *callbacklib_names[total_callbacks] = {
     "handle_overload",
     "missing_character",
     "process_character",
+    "linebreak_quality",
+    "paragraph_pass",
+    "handle_uleader",
+    "italic_correction",
 };
 
 /*tex
@@ -307,7 +314,7 @@ static int callbacklib_aux_run(lua_State *L, int id, int special, const char *va
                         return tex_formatted_error("callback", "string expected, not: %s\n", lua_typename(L, t));
                 }
                 break;
-            case callback_result_key:
+            case callback_result_s_key:
                 switch (t) {
                     case LUA_TNIL:
                         *va_arg(vl, int *) = 0;
@@ -342,6 +349,16 @@ static int callbacklib_aux_run(lua_State *L, int id, int special, const char *va
                         return tex_formatted_error("callback", "string, false or nil expected, not: %s\n", lua_typename(L, t));
                 }
                 break;
+            case callback_result_i_key:
+                switch (t) {
+                    case LUA_TNUMBER:
+                        *va_arg(vl, int *) = lmt_tointeger(L, nres);
+                        break;
+                    default:
+                     /* *va_arg(vl, int *) = 0; */ /*tex We keep the value! */
+                        break;
+                }
+                break;
             default:
                 return tex_formatted_error("callback", "invalid value type returned\n");
         }
