@@ -73,10 +73,29 @@ static int fontlib_aux_count_hash_items(lua_State *L)
 
 */
 
-# define set_numeric_field_by_index(target,name,dflt) \
-    lua_push_key(name); \
-    target = (lua_rawget(L, -2) == LUA_TNUMBER) ? lmt_roundnumber(L, -1) : dflt ; \
-    lua_pop(L, 1);
+# if (1) 
+
+    # define set_numeric_field_by_index(target,name,dflt) \
+        lua_push_key(name); \
+        target = (lua_rawget(L, -2) == LUA_TNUMBER) ? lmt_roundnumber(L, -1) : dflt ; \
+        lua_pop(L, 1);
+
+# else 
+
+    # define set_numeric_field_by_index(target,name,dflt) \
+        lua_push_key(name); \
+        if (lua_rawget(L, -2) == LUA_TNUMBER) { \
+            target = lmt_roundnumber(L, -1); \
+            lua_pop(L, 1); \
+            lua_push_key(name); \
+            lua_push_integer(L, target); \
+            lua_rawset(L, -3); \
+        } else { \
+            target = dflt ; \
+            lua_pop(L, 1); \
+        } 
+
+# endif 
 
 # define set_boolean_field_by_index(target,name,dflt) \
     lua_push_key(name); \
@@ -156,7 +175,14 @@ static void fontlib_aux_read_lua_parameters(lua_State *L, int f)
                             tex_set_font_parameter(f, em_width_code, value);
                         } else if (lua_key_eq(s, extraspace)) {
                             tex_set_font_parameter(f, extra_space_code, value);
+                        } else { 
+                            break;
                         }
+// lua_pop(L, 1);
+// lua_push_string(L, s);
+// lua_push_integer(L, value);
+// lua_rawset(L, -3);
+// return;
                     }
                     break;
                 case LUA_TNUMBER:
@@ -263,14 +289,20 @@ static void fontlib_aux_font_char_from_lua(lua_State *L, halfword f, int i, int
         set_charinfo_depth(co, target);
         set_numeric_field_by_index(target, italic, 0);
         set_charinfo_italic(co, target);
-        set_numeric_field_by_index(target, expansion, 1000);
+        set_numeric_field_by_index(target, expansion, scaling_factor);
         set_charinfo_expansion(co, target);
         set_numeric_field_by_index(target, compression, target);
         set_charinfo_compression(co, target);
         set_numeric_field_by_index(target, leftprotrusion, 0);
-        set_charinfo_leftprotrusion(co, target);
+        if (target) { 
+            set_font_text_control(f, text_control_left_protrusion);
+            set_charinfo_leftprotrusion(co, target);
+        }
         set_numeric_field_by_index(target, rightprotrusion, 0);
-        set_charinfo_rightprotrusion(co, target);
+        if (target) { 
+            set_font_text_control(f, text_control_right_protrusion);
+            set_charinfo_rightprotrusion(co, target);
+        }
         if (has_math) {
             tex_char_malloc_mathinfo(co);
             set_numeric_field_by_index(target, smaller, 0);
@@ -563,6 +595,7 @@ static int lmt_font_from_lua(lua_State *L, int f)
         if (! no_math) {
             fontlib_aux_read_lua_math_parameters(L, f);
         }
+        lmt_font_state.fonts[f]->weight = 65.536 * lmt_font_state.fonts[f]->design_size / lmt_font_state.fonts[f]->size;
         /*tex The characters. */
         lua_push_key(characters);
         if (lua_rawget(L, -2) == LUA_TTABLE) {
@@ -631,15 +664,15 @@ static int lmt_font_from_lua(lua_State *L, int f)
                 if (fstep > 0) {
                     int fstretch = 0;
                     int fshrink = 0;
-                    if (fstep > 100) {
-                        fstep = 100;
+                    if (fstep > max_font_adjust_step) {
+                        fstep = max_font_adjust_step;
                     }
                     set_numeric_field_by_index(fshrink, shrink, 0);
                     set_numeric_field_by_index(fstretch, stretch, 0);
                     if (fshrink < 0) {
                         fshrink = 0;
-                    } else if (fshrink > 500) {
-                        fshrink = 500;
+                    } else if (fshrink > max_font_adjust_shrink_factor) {
+                        fshrink = max_font_adjust_shrink_factor;
                     }
                     fshrink -= (fshrink % fstep);
                     if (fshrink < 0) {
@@ -647,8 +680,8 @@ static int lmt_font_from_lua(lua_State *L, int f)
                     }
                     if (fstretch < 0) {
                         fstretch = 0;
-                    } else if (fstretch > 1000) {
-                        fstretch = 1000;
+                    } else if (fstretch > max_font_adjust_stretch_factor) {
+                        fstretch = max_font_adjust_stretch_factor;
                     }
                     fstretch -= (fstretch % fstep);
                     if (fstretch < 0) {
@@ -657,6 +690,7 @@ static int lmt_font_from_lua(lua_State *L, int f)
                     set_font_step(f, fstep);
                     set_font_max_stretch(f, fstretch);
                     set_font_max_shrink(f, fshrink);
+                    set_font_text_control(f, text_control_expansion);
                 }
             } else {
                 tex_formatted_warning("font", "lua-loaded font '%d' with name '%s' has no characters", f, font_name(f));
@@ -737,6 +771,48 @@ static int lmt_characters_from_lua(lua_State *L, int f)
     return 1;
 }
 
+static int lmt_quality_from_lua(lua_State *L, int f)
+{
+    lua_push_key(characters);
+    if (lua_rawget(L, -2) == LUA_TTABLE) {
+        int first = font_first_character(f);
+        int last = font_last_character(f);
+        lua_pushnil(L);
+        while (lua_next(L, -2)) {
+            if (lua_type(L, -2) == LUA_TNUMBER) {
+                int i = lmt_tointeger(L, -2);
+                if ((i >= first && i <= last) && tex_char_exists(f, i) && lua_istable(L, -1)) {
+                    int target; 
+                    charinfo *co = tex_get_charinfo(f, i);
+                    if (! has_charinfo_tag(co, expansion_tag)) {
+                        set_numeric_field_by_index(target, expansion, scaling_factor);
+                        set_charinfo_expansion(co, target);
+                        set_numeric_field_by_index(target, compression, target);
+                        set_charinfo_compression(co, target);
+                        set_charinfo_tag(co, expansion_tag);
+                    }
+                    if (! has_charinfo_tag(co, protrusion_tag)) {
+                        set_numeric_field_by_index(target, leftprotrusion, 0);
+                        if (target) { 
+                            set_font_text_control(f, text_control_left_protrusion);
+                            set_charinfo_leftprotrusion(co, target);
+                        }
+                        set_numeric_field_by_index(target, rightprotrusion, 0);
+                        if (target) { 
+                            set_font_text_control(f, text_control_right_protrusion);
+                            set_charinfo_rightprotrusion(co, target);
+                        }
+                        set_charinfo_tag(co, protrusion_tag);
+                    }
+                }
+            }
+            lua_pop(L, 1);
+        }
+        lua_pop(L, 1);
+    }
+    return 1;
+}
+
 /*tex
 
    The font library has helpers for defining the font and setting or getting the current font.
@@ -795,6 +871,24 @@ static int fontlib_addcharacters(lua_State *L)
     return 0;
 }
 
+static int fontlib_addquality(lua_State *L)
+{
+    int i = lmt_checkinteger(L, 1);
+    if (i) {
+        if (tex_is_valid_font(i)) {
+            if (! has_font_text_control(i, text_control_quality_set)) {
+                luaL_checktype(L, 2, LUA_TTABLE);
+                lua_settop(L, 2);
+                lmt_quality_from_lua(L, i);
+                set_font_text_control(i, text_control_quality_set);
+            }
+        } else {
+            return luaL_error(L, "invalid font id %d passed", i);
+        }
+    }
+    return 0;
+}
+
 /*tex |font.define(table)| */
 
 static int fontlib_define(lua_State *L)
@@ -825,7 +919,7 @@ static int fontlib_id(lua_State *L)
     if (lua_type(L, 1) == LUA_TSTRING) {
         size_t l;
         const char *s = lua_tolstring(L, 1, &l);
-        int cs = tex_string_locate(s, l, 0);
+        int cs = tex_string_locate_only(s, l);
         int f = -1;
         if (cs == undefined_control_sequence || cs == undefined_cs_cmd || eq_type(cs) != set_font_cmd) {
             lua_pushliteral(L, "not a valid font csname");
@@ -903,7 +997,7 @@ static int fontlib_getmathspec(lua_State *L)
     if (lua_type(L, 1) == LUA_TSTRING) {
         size_t lname = 0;
         const char *name = lua_tolstring(L, 1, &lname);
-        halfword cs = tex_string_locate(name, lname, 0);
+        halfword cs = tex_string_locate_only(name, lname);
         if (eq_type(cs) == mathspec_cmd) {
             halfword ms = eq_value(cs);
             if (ms) {
@@ -923,7 +1017,7 @@ static int fontlib_getfontspec(lua_State *L)
     if (lua_type(L, 1) == LUA_TSTRING) {
         size_t lname = 0;
         const char *name = lua_tolstring(L, 1, &lname);
-        halfword cs = tex_string_locate(name, lname, 0);
+        halfword cs = tex_string_locate_only(name, lname);
         if (eq_type(cs) == fontspec_cmd) {
             halfword fs = eq_value(cs);
             if (fs) {
@@ -931,7 +1025,9 @@ static int fontlib_getfontspec(lua_State *L)
                 lua_pushinteger(L, font_spec_scale(fs));
                 lua_pushinteger(L, font_spec_x_scale(fs));
                 lua_pushinteger(L, font_spec_y_scale(fs));
-                return 4;
+                lua_pushinteger(L, font_spec_slant(fs));
+                lua_pushinteger(L, font_spec_weight(fs));
+                return 6;
             }
         }
     }
@@ -958,11 +1054,24 @@ static int fontlib_getmathindex(lua_State *L) {
     return 2;
 }
 
+static int fontlib_xscaled(lua_State *L)
+{
+    lua_pushinteger(L, tex_font_x_scaled(lmt_optroundnumber(L, 1, 1)));
+    return 1;
+}
+
+static int fontlib_yscaled(lua_State *L)
+{
+    lua_pushinteger(L, tex_font_y_scaled(lmt_optroundnumber(L, 1, 1)));
+    return 1;
+}
+
 static const struct luaL_Reg fontlib_function_list[] = {
     { "current",       fontlib_current       },
     { "max",           fontlib_max           },
     { "setfont",       fontlib_setfont       },
     { "addcharacters", fontlib_addcharacters },
+    { "addquality",    fontlib_addquality    },
     { "define",        fontlib_define        },
     { "nextid",        fontlib_nextid        },
     { "id",            fontlib_id            },
@@ -971,6 +1080,8 @@ static const struct luaL_Reg fontlib_function_list[] = {
     { "getfontspec",   fontlib_getfontspec   },
     { "getmathspec",   fontlib_getmathspec   },
     { "getmathindex",  fontlib_getmathindex  },
+    { "xscaled",       fontlib_xscaled       },
+    { "yscaled",       fontlib_yscaled       },
     { NULL,            NULL                  },
 };
 
