@@ -83,11 +83,22 @@
     # include <direct.h>
     # include <windows.h>
     # include <io.h>
+    # include <fileapi.h>
     # include <sys/locking.h>
     # include <sys/utime.h>
     # include <fcntl.h>
 
-    # define MY_MAXPATHLEN MAX_PATH
+    /* 
+        Todo MS Windows: By default, the name is limited to MAX_PATH characters. To extend this 
+        limit to 32,767 wide characters, prepend "\\?\" to the path. For more information, see 
+        Naming Files, Paths, and Namespaces.
+    */
+
+    # ifdef MAX_PATH
+        # define MY_MAXPATHLEN MAX_PATH
+    # else 
+        # define MY_MAXPATHLEN 255
+    # endif 
 
 # else
 
@@ -100,7 +111,11 @@
     # include <utime.h>
     # include <sys/param.h>
 
-    # define MY_MAXPATHLEN MAXPATHLEN
+    # ifdef MAXPATHLEN
+        # define MY_MAXPATHLEN MAXPATHLEN
+    # else 
+        # define MY_MAXPATHLEN 255
+    # endif 
 
 # endif
 
