@@ -52,6 +52,38 @@ insert_state_info lmt_insert_state = {
     .storing     = 0,
 };
 
+typedef enum saved_insert_entries {
+    saved_insert_index_entry  = 0, /* value_1 */
+    saved_insert_n_of_records = 1,
+} saved_insert_entries;
+
+# define saved_insert_index saved_value_1(saved_insert_index_entry)
+
+inline static void saved_inserts_initialize(void)
+{
+    saved_type(0) = saved_record_0;
+    saved_record(0) = insert_save_type;
+}
+
+void tex_show_insert_group(void)
+{
+    tex_print_str_esc("insert");
+    tex_print_int(saved_insert_index);
+}
+
+int tex_show_insert_record(void)
+{
+    tex_print_str("insert ");
+    switch (save_type(lmt_save_state.save_stack_data.ptr)) { 
+       case saved_record_0:
+            tex_print_format("index %i", saved_insert_index);
+            break;
+        default: 
+            return 0;
+    }
+    return 1;
+}
+
 void tex_initialize_inserts(void)
 {
     insert_record *tmp = aux_allocate_clear_array(sizeof(insert_record), lmt_insert_state.insert_data.minimum, 1);
@@ -296,7 +328,7 @@ void tex_wipe_insert(halfword i) {
 
 halfword lmt_get_insert_distance(halfword i, int slot)
 {
-    int callback_id = lmt_callback_defined(build_page_insert_callback);
+    int callback_id = lmt_callback_defined(insert_distance_callback);
     if (callback_id != 0) {
         halfword replacement = null;
         lmt_run_callback(lmt_lua_state.lua_instance, callback_id, "dd->N", i, slot, &replacement);
@@ -360,7 +392,7 @@ halfword tex_scan_insert_index(void)
             }
             break;
         case class_insert_mode:
-            index = tex_scan_int(0, NULL);
+            index = tex_scan_integer(0, NULL);
             if (! tex_valid_insert_id(index)) {
                 index = 0;
             }
@@ -444,19 +476,20 @@ void tex_undump_insert_data(dumpstream f) {
 
 void tex_run_insert(void)
 {
-    tex_set_saved_record(saved_insert_item_index, insert_index_save_type, 0, tex_scan_insert_index());
-    lmt_save_state.save_stack_data.ptr += saved_insert_n_of_items;
+    saved_inserts_initialize();
+    saved_insert_index = tex_scan_insert_index();
+    lmt_save_state.save_stack_data.ptr += saved_insert_n_of_records;
     tex_new_save_level(insert_group);
     tex_scan_left_brace();
     tex_normal_paragraph(insert_par_context);
     tex_push_nest();
     cur_list.mode = internal_vmode;
-    cur_list.prev_depth = ignore_depth_criterium_par;
+    cur_list.prev_depth = ignore_depth_criterion_par;
 }
 
 void tex_finish_insert_group(void)
 {
-    if (! tex_wrapped_up_paragraph(insert_par_context)) {
+    if (! tex_wrapped_up_paragraph(insert_par_context, 0)) {
         halfword p, q; /*tex for short-term use */
         scaled d;      /*tex holds |split_max_depth| in |insert_group| */
         halfword f;    /*tex holds |floating_penalty| in |insert_group| */
@@ -465,17 +498,17 @@ void tex_finish_insert_group(void)
         d = split_max_depth_par;
         f = floating_penalty_par;
         tex_unsave();
-        lmt_save_state.save_stack_data.ptr -= saved_insert_n_of_items;
-     // p = tex_vpack(node_next(cur_list.head), 0, packing_additional, max_dimen, direction_unknown);
+        lmt_save_state.save_stack_data.ptr -= saved_insert_n_of_records;
+     // p = tex_vpack(node_next(cur_list.head), 0, packing_additional, max_dimension, direction_unknown);
      // /* we don't do this: */
-     // /* p = tex_filtered_vpack(node_next(cur_list.head), 0, packing_additional, max_dimen, insert_group, direction_unknown, 0, 0); */
+     // /* p = tex_filtered_vpack(node_next(cur_list.head), 0, packing_additional, max_dimension, insert_group, direction_unknown, 0, 0); */
      // /* because it can induce loops. */
      // tex_pop_nest();
         p = node_next(cur_list.head);
         tex_pop_nest();
-        p = tex_vpack(p, 0, packing_additional, max_dimen, direction_unknown, holding_none_option);
+        p = tex_vpack(p, 0, packing_additional, max_dimension, direction_unknown, holding_none_option, NULL);
         {
-            halfword index = saved_value(saved_insert_item_index);
+            halfword index = saved_insert_index;
             halfword insert = tex_new_node(insert_node, 0);
             halfword maxdepth = tex_get_insert_maxdepth(index);
             halfword floating = tex_get_insert_penalty(index);
@@ -499,8 +532,8 @@ void tex_finish_insert_group(void)
             if (tracing_inserts_par > 0) {
                 tex_begin_diagnostic();
                 tex_print_levels();
-                tex_print_format("[insert: setting, index %i, height %D, penalty %i]",
-                    index, insert_total_height(insert), pt_unit, insert_float_cost(insert));
+                tex_print_format("[insert: setting, index %i, height %p, penalty %i]",
+                    index, insert_total_height(insert), insert_float_cost(insert));
                 if (tracing_inserts_par > 1) {
                     tex_print_node_list(insert_list(insert), "insert", show_box_depth_par, show_box_breadth_par);
                 }
@@ -509,10 +542,7 @@ void tex_finish_insert_group(void)
         }
         /* we never do the callback ... maybe move it outside */
         if (lmt_nest_state.nest_data.ptr == 0) {
-            if (! lmt_page_builder_state.output_active) {
-                lmt_page_filter_callback(insert_page_context, 0);
-            }
-            tex_build_page();
+            tex_build_page(insert_page_context, 0);
         }
     }
 }
