@@ -364,7 +364,8 @@ void tex_clear_patterns(struct tex_language *lang)
 
 void tex_load_tex_patterns(halfword curlang, halfword head)
 {
-    char *s = tex_tokenlist_to_tstring(head, 1, NULL, 0, 0, 0, 0);
+    /*tex We might want single hashes. */
+    char *s = tex_tokenlist_to_tstring(head, 1, NULL, 0, 0, 0, 0, 1); /* single hashes */
     if (s) {
         tex_load_patterns(tex_get_language(curlang), (unsigned char *) s);
     }
@@ -540,7 +541,7 @@ void tex_clear_hyphenation(struct tex_language *lang)
 
 void tex_load_tex_hyphenation(halfword curlang, halfword head)
 {
-    char *s = tex_tokenlist_to_tstring(head, 1, NULL, 0, 0, 0, 0);
+    char *s = tex_tokenlist_to_tstring(head, 1, NULL, 0, 0, 0, 0, 1); /* single hashes */
     if (s) {
         tex_load_hyphenation(tex_get_language(curlang), (unsigned char *) s);
     }
@@ -965,7 +966,7 @@ inline static halfword tex_aux_is_hyphen_char(halfword chr)
     if (tex_get_hc_code(chr)) {
         return tex_get_hc_code(chr);
     } else if (chr == ex_hyphen_char_par) {
-        return ex_hyphen_char_par;
+        return chr;
     } else {
         return null;
     }
